function mainCerebellar(directory)


% Main of Cerebellar Climbing Fibers

% Data Information
% Experimental Procedures:
% Dataset owner: GA Ascoli, Molecular Neuroscience Department, Krasnow
%       Institute for Advanced Study, George Mason University (Imaging and Reconstructions);
%       I Sugihara, Department of Systems Neurophysiology, Tokyo Medial and Dental University,
%       Graduate School of Medicine (Histology and Labeling)
% Species: Rat
% Strain: Long-Evans
% Nervous System Region: Cerebellar Cortex
% Fiber type: Axon Terminals
% Labeling Method: Biotinylated Dextran Amine (Anterograde)
% Image Acquisition Method: Transmitted Light Brightfield
% Tracing Method: Neurolucida (Colchester, VT)
% Objective Lens: 100x oil (NA=1.3) with 2.0x zoom
% See Jefferis GS et al., 2007, Cell Vol 128(6) for general information related to the data
% Z distance between successive images within an image stack: 8.80 pixels

% Spacings : X&Y = 0.0033333 inches, Z = 1 inches
%%%%%%%%%%%%%%%%%
% Manual Reconstruction Information
% CF_1 (X,Y,Z): (4882,1797,19); Training tree; Manual reconstruction provided
% CF_2 (X,Y,Z): (818,332,6); Scored tree; Manual reconstruction not provided

%%%%%%%%%%%%%%%%%
% NOTE: x,y,z (above) is defined for image domain, in order to make
% notation consistent with Matlab, I converted the indexces to X,Y,Z, so in
% the code the order is as follows
%               --- Z = x-->
%              /|
%             / |
%       Y = y/  |
%               |
%            X = z


addpath(genpath([cd '/mfiles']))
beginn = 0;
if (beginn)
    clear all;close all;clc
    % Keep parameters inside params
    downFactor = 9;
    params.xdown = downFactor;
    params.ydown = downFactor;
    params.dim.x = round(5100/downFactor);
    params.dim.y = round(3102/downFactor);
    % # of images:
    % OP1:60;OP2:88;OP3:62;OP4:67;OP5:76;OP6:101;
    
    params.dataset = 2;
    params.zdist = 1;%3.03;
    params.spacing.xyz = [0.3296485, 0.3296485, 1];
    
    switch params.dataset
        %     convert seed points to matlab indecies: X-Y-Z = z-y-x
        case 1
            seedxyz = [4882,1797,19];
            seed = [seedxyz(3) seedxyz(2) seedxyz(1)];
            seed = seed + [1 0 0];
            params.dim.z = 34;
        case 2
            
            for i=1
                seedxyz = [818,332,6];
                seed = [seedxyz(3) seedxyz(2) seedxyz(1)];
                seed = seed + [1 0 0];
                params.dim.z = 28;
                
                params.tangVecprev = [0;1;0];   %initial direction of pc
                params.sptSig = 5;
                params.colSig = .5;
                params.kernelWeightFactor = .2; %power of the weight term in tracing step
                params.Factors.neig=2;  % weights of vectorial tracing
                params.Factors.vprev=1;
                params.Tracing.lenTang=1;    %tangent vec
                params.Tracing.lenVal=1;  %color
                params.branchFibStart = 4;
                params.Bif.varThr = false;
                params.Bif.remLowThr = .5;
                params.Bif.Rmin = 4;
                params.Bif.Rmax = 14;
                params.Bif.clusterSize = 3;
                params.Bif.ClusterDistThr = 5;
                params.Bif.centroidImtensityThr = .3;
                params.Bif.intensityThr = .05;
                params.remIntensityThr = .15;
                params.IntensityThr = .1;
                
                params.MS.sptSig = 3;    %to find bifurcations use a smaller spt. kernel
                params.MS.colSig = .5;    %to find bifurcations use a smaller spt. kernel
                params.varSig = false;
                
                % for 17 => 7; sum(cv) = 27 => 2.5;  .45*sum(cv)+14.65
                % for 17 => 5; sum(cv) = 27 => 2.5;  .25*sum(cv)+9.25
                % ax+b=y;
                params.varSigMax = 7;
                params.varSigMin = 2.5;
                params.varSigMinBound = 1.5;
                x=[17 27];
                y=[params.varSigMax params.varSigMin];
                a=(y(2)-y(1))/(x(2)-x(1));
                b=y(1)-a*x(1);
                params.varSigCoef = [a,b];
                
                params.smoothImages = false;
                
                params.forePerc = 100;
                params.filt = false;
                params.Bif.traj.Lthr = 1.5;
                params.Bif.traj.bthr = -.4;
                
                
            end
    end
    params.seed = seed;
    params.directory = 'C:/ERHAN/WORK/DATA/DIADEM/Cerebellar Climbing Fibers/Image Stacks/Qualifier Image Stack/Image Stack CF_2';
    % Load Images
    IM=zeros(params.dim.z,params.dim.y,params.dim.x,3);
    dim = size(IM);
    parfor i=1:params.dim.z
        i
        loadName = sprintf('%s/%02d.tif',params.directory,i);
        imtemp = im2double(imread(loadName));
        %         IM(i,:,:) = adapthisteq(imresize(imtemp(:,:,2),1/9)); % take only green channel and downsample with 9
        IM(i,:,:,:) = (imresize(imtemp,1/9)); % take only green channel and downsample with 9
    end
    
    
    %     for i=1:dim(2)
    %         IM2(:,i,:,:) = imresize(squeeze(IM(:,i,:)),[round(dim(1)) dim(3)]);
    %     end
    
    Imori = IM;
    Im = IM;
    data = Im;
    clearvars -except Im IM dim params data Imori
    save DATA_Cereb
    %%
    
    [dataMIP indMIP]= min(Im,[],1);
    dataMIP = squeeze(dataMIP);
    indMIP =squeeze(indMIP);
    
    for i=1:28
        G(i,:,:)=colorGrad(squeeze(Im(i,:,:,:)));
    end
    %%
    % create mask
    se = strel('disk',5);
    mask = dataMIP(:,:,1) > .9;
    % figure, imshow(mask)
    mask = imdilate(mask,se);
    % figure, imshow(mask)
    
    
    [GMIP indGMIP]= max(G,[],1);
    GMIP = squeeze(GMIP);
    GMIP(mask)=0;
    indGMIP =squeeze(indGMIP);
    
    GMIPcl = imclose(GMIP,se);
    GMIPmask = GMIPcl>max(GMIPcl(:))*.2;
    BW = zeros(size(GMIP));
    
    MIPgra = colorGrad(dataMIP,0,'2D');
    MIPgra(mask)=0;
    
    CC = bwconncomp(GMIPmask);
    numPixels = cellfun(@numel,CC.PixelIdxList);
    [biggest,idx] = max(numPixels);
    BW(CC.PixelIdxList{idx}) = 1;
    
    figure, imshow(dataMIP)
    figure, imshow(GMIP)
    figure, imshow(GMIPcl)
    figure, imshow(GMIPmask)
    figure, imshow(MIPgra,[])
    figure, imshow(BW)
    %%
    se = strel('disk',5);
    for i=1:28
        Gcl(i,:,:)=imclose(squeeze(G(i,:,:,:)),se).*BW;
    end
    figure(30), for i=1:28;imshow(squeeze(Gcl(i,:,:))>.1);title(num2str(i));drawnow;end
    
    %%
    % 2Rz = 12 & 2Rx-y = 6 => downsample z with 2
        params.zdown = 2;
        params.dim.z = params.dim.z/params.zdown;

    Gcl2 = zeros(size(Gcl)./[2 1 1]);
    data = zeros([size(Gcl2) 3]);
    for i=1:size(Gcl,2)
        Gcl2(:,i,:)=imresize(squeeze(Gcl(:,i,:)),[size(Gcl,1)/2 size(Gcl,3)],'nearest');
        data(:,i,:,:)=imresize(squeeze(Imori(:,i,:,:)),[size(Gcl,1)/2 size(Gcl,3)]);
    end
    figure(31), for i=1:14;imshow(squeeze(Gcl2(i,:,:))>.1);title(num2str(i));drawnow;end
    dim = size(Gcl2);
    %%
    [Y X Z]= meshgrid(1:dim(2),1:dim(1),1:dim(3));
    params.zdist = 1;
    indVec = [round(params.zdist*X(:)),Y(:),Z(:)];
    dataVec = [indVec reshape(data,numel(data)/3,3)];
    foreIm = Gcl2>.1;
    indforeIm = find(foreIm);
    imVec = dataVec(foreIm,:);
    p = imVec(:,1:3);
    c = imVec(:,4:6);
    f = [p c];
    Imorid = data;
    clearvars -except f foreIm dim data params dataVec Imori Imorid indforeIm
    %%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % Create KNN graph
    ptrtree=BuildGLTree(f(:,1:3));
    
    k=250;  %500->R~5, 250->R~4
    [KNNG,distances]=KNNSearch(f(:,1:3),f(:,1:3),ptrtree,k);
    N = length(KNNG);
    IndI = zeros(1,length(foreIm));
    IndI(indforeIm) = 1:N;
    
    ForImage = reshape(foreIm,dim);
    
    
    %%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %     fori = f;
    %     f_filt = f;
    %     parfor ind=1:N
    %         if ~mod(ind,1000)
    %             ind
    %         end
    %         indknn = KNNG(ind,1:k);
    %         f_filt(ind,end) = max(f(indknn,end));
    %     end
    %     params.f_filt=f_filt;
    %
    %     IT =  zeros(dim(1),dim(2),dim(3));
    %     IT(find(IndI)) = f_filt(:,end);
    %     figure, imshow(squeeze(data(4,:,:)))
    %%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    % vocal covariances
    params.varSig = 0;
    if params.varSig
        
        parfor i=1:length(f)
            cv=(diag(cov(f(KNNG(i,:),1:params.featDimP))));
            varSptSig(i)=params.varSigCoef(1)*sum(cv)+params.varSigCoef(2); %sum(cv) = 17 => 7; sum(cv) = 30 => 2.5;
        end
        % k=500=> R~7, siunce sum([1:2:11].^2+13^2)=455~=500
        % from training:
        % sum(cv) = 17 => 5; sum(cv) = 30 => 2.5;
        varSptSig(varSptSig>params.varSigMax)=params.varSigMax;
        varSptSig(varSptSig<params.varSigMinBound)=params.varSigMinBound;
        params.varSptSig=varSptSig;
    else
        
    end
    
    
    %%
    %     f = fori;
    % indexOfSeed=find(all(f(:,1:3)==repmat(round(params.seed.*[params.zdist 1 1]),length(f),1),2));
    %     indexOfSeed=find(all(f(:,1:3)==repmat(round(params.seed.*[params.zdist 1 1]),length(f),1),2));
    %     I=zeros(3,length(KNNG));
    Inext=zeros(3,length(KNNG));
    indknn_ = 1:k;
    
    params.featDimP = 3;
    params.featDimC = 3;
    params.featDim = params.featDimC + params.featDimP;
    
    params.ani = 0;
    params.invCi = [];
    params.invCxi=[];
    params.knnNum = k;
    params.fullH = 0;   %0-> uses only Hpp, 1: full H
    
    %     params.sptSigx = params.sptSig;
    %     params.sptSigy = params.sptSig;
    %     params.sptSigz = params.sptSig;
    params.scale = 1;
    params.normalUpdateThr = .5;
    
    
    if params.varSig
        varSpt=params.varSptSig;
        % Calculate initial PC projections using variable size kernels
        projPoints2 = zeros(params.featDimP,N);
        projPointsf2 = zeros(params.featDimP,N);
        projVecs2 = zeros(params.featDimP,params.featDimP,N);
        Covx2 = zeros(3,3,N);
        tic
        parfor indexOfVoxel=1:N
            sptSig = varSpt(indexOfVoxel)*1;
            if ~mod(indexOfVoxel,1000)
                indexOfVoxel
            end
            indknn = KNNG(indexOfVoxel,1:k);
            f_i = f(indknn,:)';
            x = f_i(:,end);
            xg = round(x(1:params.featDimP));
            
            [pcUpdate_normal subSpace pctang SI vals] = pcProj(x,f_i,params,f_i(4,:).^1,sptSig);
            proj_normalD = pcUpdate_normal;%./[params.zdist;1;1];
            it = 1;
            ind_it=1;
            
            if any(abs(proj_normalD) >params.normalUpdateThr)
                continue;
            else
                %             while any(abs(pcUpdate_normal./[params.zdist;1;1])>.05) &  ind_it & it<=100
                while any(abs(pcUpdate_normal)>.05) &  ind_it & it<=100
                    [pcUpdate_normal subSpace pctang SI vals] = pcProj(x,f_i,params,f_i(4,:).^1,sptSig);
                    proj_normalD = proj_normalD + pcUpdate_normal./[params.zdist;1;1];
                    x(1:params.featDimP) = x(1:params.featDimP) + pcUpdate_normal;
                    %                 hold on, plot3(x(3),x(2),x(1),'g.')
                    xg = round(x(1:params.featDimP));
                    xg = max([1;1],xg);
                    xg = min(dim(:),xg);
                    ind_it = IndI(sub2ind(dim(1:params.featDimP),xg(1,:),xg(2,:)));
                    if ~ind_it %| any(abs(pcUpdate_normal./[params.zdist;1;1]) >params.normalUpdateThr)
                        continue;
                    end
                    x(params.featDimP+1:end) = f(ind_it,(params.featDimP+1:end));
                    
                    indknn = KNNG(ind_it,1:k);
                    f_i = f(indknn,:)';
                    it = it+1;
                end
                
                if it>100 | ~ind_it
                    continue;
                end
                
                projPoints2(:,indexOfVoxel) = xg;
                projPointsf2(:,indexOfVoxel) = x(1:params.featDimP);
                projVecs2(:,:,indexOfVoxel) = [pctang(:) subSpace];
                temp = inv(SI);
                Covx2(:,:,indexOfVoxel) = sign(temp(1))*temp;
            end
        end
        toc
    else
        %%
        % Calculate initial PC projections
        projPoints1 = zeros(params.featDimP,N);
        projPointsf1 = zeros(params.featDimP,N);
        projVecs1 = zeros(params.featDimP,params.featDimP,N);
        Covx1 = zeros(params.featDimP,params.featDimP,N);
        %     matlabpool(4)
        tic
        for indexOfVoxel=1:N
            sptSig = params.sptSig;
            if ~mod(indexOfVoxel,1000)
                indexOfVoxel
            end
            indknn = KNNG(indexOfVoxel,1:k);
            f_i = f(indknn,:)';
            x = f_i(:,end);
            xg = round(x(1:params.featDimP));
            
            wkernel = sum(f_i(params.featDimP+1:end,:),1); wkernel = wkernel/sum(wkernel);
            wkernel = 1-wkernel;
            
            [pcUpdate_normal subSpace pctang SI vals] = pcProj(x,f_i,params,wkernel,sptSig);
            proj_normalD = pcUpdate_normal;%./[params.zdist;1;1];
            it = 1;
            ind_it=1;
            
            if any(abs(proj_normalD) >params.normalUpdateThr) %| params.f3(indexOfVoxel,end)<params.Bif.traj.bthr
                continue;
            else
                %             while any(abs(pcUpdate_normal./[params.zdist;1;1])>.05) &  ind_it & it<=100
                while any(abs(pcUpdate_normal)>.05) &  ind_it & it<=100
                    wkernel = sum(f_i(params.featDimP+1:end,:),1); wkernel = wkernel/sum(wkernel);
                    wkernel = 1-wkernel;
                    [pcUpdate_normal subSpace pctang SI vals] = pcProj(x,f_i,params,wkernel,sptSig);
                    proj_normalD = proj_normalD + pcUpdate_normal;
                    x(1:params.featDimP) = x(1:params.featDimP) + pcUpdate_normal;
                    %                 hold on, plot3(x(3),x(2),x(1),'g.')
                    xg = round(x(1:params.featDimP));
                    xg = max([1;1;1],xg);
                    xg = min(dim(:),xg);
                    ind_it = IndI(sub2ind((dim(1:params.featDimP)),xg(1,:),xg(2,:),xg(3,:)));
                    if ~ind_it %| any(abs(pcUpdate_normal./[params.zdist;1;1]) >params.normalUpdateThr)
                        continue;
                    end
                    x(params.featDimP+1:end) = f(ind_it,(params.featDimP+1:end));
                    
                    indknn = KNNG(ind_it,1:k);
                    f_i = f(indknn,:)';
                    it = it+1;
                end
                
                if it>100 | ~ind_it
                    continue;
                end
                
                projPoints1(:,indexOfVoxel) = xg;
                projPointsf1(:,indexOfVoxel) = x(1:params.featDimP);
                projVecs1(:,:,indexOfVoxel) = [pctang(:) subSpace];
                temp = inv(SI);
                Covx1(:,:,indexOfVoxel) = sign(temp(1))*temp;
            end
        end
        toc
    end
    % save vs7_PC_knn300_.mat
    %%
    % Im=data/max(data(:));
    % Im2(:,:,:,1)=Im;
    % Im2(:,:,:,2)=Im;
    % Im2(:,:,:,3)=Im;
    Im2 = Imorid;
    
    if 1
        if ~params.varSig
            indPC = find(any(projPointsf1));
            projPointsSet = projPointsf1(:,indPC);
            figure,
            imshow(squeeze(Im2(7,:,:,:)),[]),hold on,
            plot3(projPointsSet(3,:),projPointsSet(2,:),projPointsSet(1,:),'g.')
            title([num2str(params.PCcol),';',num2str(params.PCspt),';',num2str(params.normalUpdateThr)])
            projPoints = projPoints1;
            projPointsf = projPointsf1;
            projVecs = projVecs1;
            Covx = Covx1;
        else
            indPC = find(any(projPointsf2));
            projPointsSet = projPointsf2(:,indPC);
            figure,
            imshow(squeeze(Im2(7,:,:,:)),[]),hold on,
            plot3(projPointsSet(3,:),projPointsSet(2,:),projPointsSet(1,:),'g.')
            title([num2str(params.colSig),';',num2str(params.sptSig),';',num2str(params.normalUpdateThr)])
            projPoints = projPoints2;
            projPointsf = projPointsf2;
            projVecs = projVecs2;
            Covx = Covx2;
        end
    end
    
end

%%  Trace a fiber

% Im=data/max(data(:));
% Im2(:,:,:,1)=Im;
% Im2(:,:,:,2)=Im;
% Im2(:,:,:,3)=Im;
Im2 = data;
bifCheck = 0;

tangVecprev = params.tangVecprev;
indexOfSeed=find(all(f(:,1:params.featDimP)==repmat(round(params.seed./[params.zdown params.ydown params.xdown]),length(f),1),2));
% if isempty(indexOfSeed)
%     indexOfSeed=find(all(f(:,1:3)==repmat(floor(params.seed.*[params.zdist 1 1]),length(f),1),2));
% end
params.visual = 0;
numOfFiber=1;
%%
initSeed = indexOfSeed;
params.initSeed = indexOfSeed;
PC=[];
PCAll = [];
params.fig=31;
params.visual = 0;
% [PC_ numOfFiber_ PCAll_ projPointsfRem_]=deal(PC,numOfFiber, PCAll, projPointsfRem);
%%
params.remCheck = 0;
params.permute=0;
% params.filt = true;
params.filt = false;
% Imm=zeros([dim 3]);
% Imm(:,:,1) = data;
% Imm(:,:,2) = data;
% Imm(:,:,3) = data;
Imm = Im2;
params.visual = 0;
% params.f2 = f2;
params.Bif.Colormodels.Back = [.714;.687;.703];
params.Bif.Colormodels.Fiber = [.36;.244;.271];
params.Bif.Colormodels.Blue = [.485;.486;.562];

params.Bif.centroidImtensityThr=.01;
params.Bif.Rmin = 5;
params.Bif.Rmax = 25;
params.Factors.neig = 4
params.IntensityThr = .15;
params.visual =0;
params.Tracing.lenTang = 10;
params.Tracing.lenVal = 3;
params.Factors.neig = 1;
params.Bif.centroidImtensityThr = 10;
[PC1 numOfFiber1 PCAll1 projPointsRem1 projPointsfRem1] = branchIt(f,indexOfSeed,tangVecprev,Imm,KNNG,IndI,PC,numOfFiber,initSeed,projPoints,projPointsf,projVecs,PCAll,params);
[PCb,numOfFiberb,PCAllb,projPointsRemb,projPointsfRemb] = deal(PC1,numOfFiber1,PCAll1,projPointsRem1,projPointsfRem1);
%%
[PC1,numOfFiber1,PCAll1,projPointsRem1,projPointsfRem1] = deal(PCb,numOfFiberb,PCAllb,projPointsRemb,projPointsfRemb);
% check any remaning points
params.remCheck = 1;

params.Bif.Rmin = 5;
params.Bif.Rmax = 25;
endTrace = 0;
params.Factors.neig = 4
iteration = 1;
while ~endTrace
    [PC1 numOfFiber1 PCAll1 projPointsRem1 projPointsfRem1 endTrace] = checkRem(f,Im2,KNNG,IndI,PC1,numOfFiber1,[],projPointsRem1,projPointsfRem1,projVecs,PCAll1,params);
    title(num2str(iteration))
    iteration = iteration+1;
    
end
%%
PCFinal = PC1;
params.indRoots = indexOfSeed;
[SWC2 S2 pickOne] = createConMatrix2(PCFinal,f,params);
%%
% SWC2=SWC;i
directory = params.directory;
for i=1:length(SWC2)
    SWC2{i}(:,[3 4 5]) = SWC2{i}(:,[3 4 5]).*repmat([ params.xdown params.xdown params.zdown],size(SWC2{i},1),1);
    SWC_ = SWC2{i};
    numOfTrace=1;
    file_1 = fopen([directory,'Qual_CF_',num2str(numOfTrace),'SWC',num2str(pickOne(i)),'.swc'],'w');
    fprintf(file_1,'# Neurolucida to SWC conversion from L-Measure.\n');
    fprintf(file_1,'%d %d %4.3f %4.3f %4.3f %d %d\n',SWC_');
    fclose(file_1);
%     isconnected(S2)
end
%%
% create connectivity matrix
% % PC= PCson;
% folder = 'C:/ERHAN/WORK/DATA/DIADEM/Olfactory Projection Fibers v2/Qualifier/';
% % PC1=PC
% [SWC1 S1 B1 wS1] = createConMatrix(PC1,f,params);
% file_1 = fopen([folder,'Qual_OF_Test_',num2str(params.dataset),'vs_S1.swc'],'w');
% fprintf(file_1,'# Neurolucida to SWC conversion from L-Measure.\n');
% fprintf(file_1,'%d %d %4.3f %4.3f %4.3f %d %d\n',SWC1');
% fclose(file_1);
% isconnected(S1)
%%

% [SWC2 S2 B2 wS2] = createConMatrix(PC2,f,params);
% file_1 = fopen([folder,'Qual_OF_Test_',num2str(params.dataset),'vs_S2.swc'],'w');
% fprintf(file_1,'# Neurolucida to SWC conversion from L-Measure.\n');
% fprintf(file_1,'%d %d %4.3f %4.3f %4.3f %d %d\n',SWC2');
% fclose(file_1);
%%

% S = (S1+S1)/2;
% B=[B1;B2];
% %%
% clc
% isconnected(S)
% isloop(S)

% [D,pred,cl] = graphtraverse(S,B(1,2),'METHOD','BFS');

